#
# Properties
#
# Copyright (c) 2001-2003 by Philipp "philiKON" von Weitershausen
#
# This software is distributed under the terms of the Mozilla Public
# License (MPL)
#

"""
$Id: Properties.py,v 1.2 2003/05/21 00:03:59 philipp Exp $
"""

from Interface import Interface

class IPropertySerializer(Interface):
    """
    Interface for property serializers.

    Reads and writes properties in tuples containing dictionaries with
    the keys 'id' (id/name of the property), 'value' (value of the
    property) and 'type' (type of the property, e.g. 'string', 'int',
    'float' etc.)
    """

    def serialize(properties):
        """
        Return the properties the 'properties' sequence in a
        serialized form
        """

    def deserialize(serials):
        """
        Return a properties sequence for 'serials'
        """

class IPropertyAdapter(Interface):
    """
    Interface for property adapters.

    Whenever a property sequence is taken as a parameter or returned,
    it should be in a form so that implementations of
    IPropertySerializer can process it.
    """

    def getProperties(obj):
        """
        Get all properties from 'obj'.
        """

    def setProperties(obj, properties):
        """
        Set properties defined in the 'properties' sequence on 'obj'.
        """

class IPropertyAdapterRegistry(Interface):
    """
    Interface for property adapter registries
    """

    def registerForClass(klass, adapter):
        """
        Register 'adapter' for class 'klass'.
        """

    def getAdapter(obj):
        """
        Return an adapter for the object 'obj'
        """
