
"""
$Id: testCVSWrapper.py,v 1.8 2003/05/02 13:58:12 philipp Exp $
"""

import os, sys, time, shutil
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
import Interface, Products
from Products.RevisionManager.CVSWrapper import CVSWrapper, CVSError
from Products.RevisionManager.interfaces.RevisionController import IRevisionController

ZopeTestCase.installProduct('RevisionManager')

MODULE = "PythonUnittest"
PREFIX = os.path.join("/tmp", "testRevisionManager-"+str(int(time.time())))
WORK_DIR = os.path.join(PREFIX, "workdir")
REPOSITORY_PATH = os.path.join(PREFIX, "repository")
TESTFILES_DIR = os.path.join(os.path.dirname(Products.RevisionManager.__file__),
                             'tests', 'testfiles')

class CVSWrapperTest(ZopeTestCase.ZopeTestCase):

    def __init__(self, method_name='runTest'):
        ZopeTestCase.ZopeTestCase.__init__(self, method_name)
        self.module = MODULE
        self.work_dir = WORK_DIR
        self.repository_path = REPOSITORY_PATH
        if not os.path.isdir(PREFIX):
            os.mkdir(PREFIX)
        if not os.path.isdir(self.work_dir):
            os.mkdir(self.work_dir)
        self.cvs_wrapper = CVSWrapper(
            module=self.module,
            work_dir=self.work_dir,
            repository_path=self.repository_path
            )

    def __del__(self):
        try:
            shutil.rmtree(PREFIX, 1)
        except OSError:
            pass
    
    def test01_verify_interface(self):
        """Verify interface implementation"""
        from Interface.Verify import verifyClass
        self.failUnless(verifyClass(IRevisionController, CVSWrapper))

    def test02_import(self):
        """Test import functionality"""
        cvs = self.cvs_wrapper
        cvs.initRepository()
        cvs.importModule("UNITTEST", "UNITTEST_1", "Python Unittest Import", import_dir=TESTFILES_DIR)
        orig_file = os.path.join(TESTFILES_DIR, 'helloworld.txt')
        orig_contents = open(orig_file).read()
        revision_file = os.path.join(self.repository_path, self.module, "helloworld.txt,v")
        revision_contents = open(revision_file).read()
        self.failUnless(revision_contents.find(orig_contents) != -1)
        
    def test03_checkout(self):
        """Test checkout functionality"""
        cvs = self.cvs_wrapper
        cvs.checkout()
        orig_file = os.path.join(TESTFILES_DIR, 'helloworld.txt')
        orig_contents = open(orig_file).read()
        checkout_file = os.path.join(self.work_dir, self.module, "helloworld.txt")
        checkout_contents = open(checkout_file).read()
        self.failUnless(checkout_contents == orig_contents)

    def test04_update_commit(self):
        """Test update/commit functionality"""
        # locally modified file
        string = "Hello CVS world!"
        cvs = self.cvs_wrapper
        filename = os.path.join(self.work_dir, self.module, "helloworld.txt")
        fp = open(filename, "w")
        fp.write(string)
        fp.close()
        expected = {'helloworld.txt':'Locally Modified'}
        self.assertEqual(cvs.update('helloworld.txt'), expected)
        cvs.commit("Unit test commit")
        self.assertEqual(cvs.update('helloworld.txt'), {})

    def test05_addfile_commit(self):
        """Test add/commit functionality"""
        # unknown file
        string = "Let it be!"
        cvs = self.cvs_wrapper
        filename = os.path.join(self.work_dir, self.module, "letitbe.txt")
        fp = open(filename, "w")
        fp.write(string)
        fp.close()
        d = {'letitbe.txt':'Unknown'}
        self.assertEqual(cvs.update(), d)
        # add file
        cvs.add("letitbe.txt")
        d = {'letitbe.txt':'Locally Added'}
        self.assertEqual(cvs.update(), d)
        cvs.commit("Unit test commit")

    def test06_tag(self):
        """Test tag"""
        cvs = self.cvs_wrapper
        cvs.tag("LETITBE", 'letitbe.txt')
        # test09 checks whether tag succeeded

    def test07_status(self):
        """Test status"""
        cvs = self.cvs_wrapper
        status = cvs.status()
        self.failUnless(status.has_key('letitbe.txt'))
        status = status['letitbe.txt']
        self.failUnless(status.has_key('filename'))
        self.assertEqual(status['filename'], 'letitbe.txt')
        self.failUnless(status.has_key('status'))
        self.failUnless(status.has_key('wrev'))
        self.failUnless(status.has_key('rrev'))
        self.failUnless(status.has_key('stag'))
        self.failUnless(status.has_key('sdate'))
        self.failUnless(status.has_key('sopt'))

    def test08_log(self):
        """Test log"""
        cvs = self.cvs_wrapper
        log = cvs.log()
        self.failUnless(log.has_key('letitbe.txt'))
        log = log['letitbe.txt']
        self.failUnless(log.has_key('tags'))
        self.failUnless(log['tags'].has_key('LETITBE'))
        self.failUnless(log.has_key('revisions'))
        rev = log['revisions']
        self.failUnless(len(rev))
        rev = rev[0]
        self.failUnless(rev.has_key('revision'))
        self.failUnless(rev.has_key('date'))
        self.failUnless(rev.has_key('author'))
        self.failUnless(rev.has_key('state'))
        self.failUnless(rev.has_key('log'))
        self.failUnless(rev.has_key('branches'))
        self.failUnless(rev.has_key('lines'))

    def test09_diff(self):
        """Test diff"""
        cvs = self.cvs_wrapper
        log = cvs.log()
        log = log['helloworld.txt']
        rev1 = log['revisions'][0]['revision']
        rev2 = log['revisions'][1]['revision']
        diff = cvs.diff('helloworld.txt', rev1, rev2)
        self.assertEqual(type(diff), type(''))
        self.failUnless(len(diff))

    def test10_merge(self):
        """Test merge"""
        cvs = self.cvs_wrapper
        # get two revisions for the file
        revisions = cvs.log('helloworld.txt')['helloworld.txt']['revisions']
        rev2 = revisions[-2]['revision']
        rev1 = revisions[-3]['revision']
        # merge the changes between them
        cvs.merge(rev1, rev2, 'helloworld.txt')
        # compare the contents with what is expected
        filename = os.path.join(self.work_dir, self.module, "helloworld.txt")
        fp = open(filename)
        contents = fp.read()
        fp.close()
        expected = "Hello World!"
        self.assertEqual(contents, expected)

    def test11_locally_modified_files(self):
        """Test getLocallyModifiedFiles"""
        cvs = self.cvs_wrapper
        # the file is still locally modified from test10
        expected = ['helloworld.txt']
        self.assertEqual(expected, cvs.getLocallyModifiedFiles())

    def test12_branch(self):
        """Test branches"""
        cvs = self.cvs_wrapper
        tag = "MYTAG"
        cvs.tag(tag, branch=1)
        cvs.update(tag=tag)
        status = cvs.status('helloworld.txt')
        self.assertEquals(tag, status['helloworld.txt']['stag'])
        cvs.update(reset_sticky=1)
        status = cvs.status('helloworld.txt')
        self.assertEquals('', status['helloworld.txt']['stag'])

    def test13_error(self):
        """Test error handling"""
        self.assertRaises(ValueError, CVSWrapper)
        self.assertRaises(ValueError, CVSWrapper,
                          repository_path='', module='', workdir='')
        #XXX check CVSError

    # real unit tests, i.e. single units

    def test_parse_status(self):
        """Test status output parsing"""
        cvs = CVSWrapper(repository_path='/home/cvs-repository',
                         module='RevisionManager',
                         work_dir='/tmp')

        fake_output = """===================================================================
File: anotherfile      	Status: Locally Modified

   Working revision:	1.1.1.1	Thu May  1 10:00:37 2003
   Repository revision:	1.1.1.1	/home/cvs-repository/revisionmanager/anotherfile,v
   Sticky Tag:		(none)
   Sticky Date:		(none)
   Sticky Options:	(none)

   Existing Tags:
	SYMBOLIC_TAG             	(revision: 1.1.1.1)
	VENDOR_TAG1              	(revision: 1.1.1.1)
	VENDOR_TAG               	(branch: 1.1.1)

"""
        status = cvs.parse_status(fake_output)
        self.assert_(status.has_key('anotherfile'))
        status = status['anotherfile']
        self.assertEqual(status['filename'], 'anotherfile')
        self.assertEqual(status['status'], 'Locally Modified')
        self.assertEqual(status['wrev'], '1.1.1.1')
        self.assertEqual(status['rpath'], '/home/cvs-repository/revisionmanager/anotherfile,v')
        self.assertEqual(status['stag'], '') 
        self.assertEqual(status['sopt'], '(none)')
        tags = status['tags']
        self.assert_(tags.has_key('SYMBOLIC_TAG'))
        self.assertEqual(tags['SYMBOLIC_TAG'], "revision: 1.1.1.1")

if __name__ == '__main__':
    framework(descriptions=1, verbosity=2)    
else:
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(CVSWrapperTest))
        return suite
