
"""
$Id: testOFSPropertyAdapter.py,v 1.2 2003/05/21 00:04:00 philipp Exp $
"""

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase

ZopeTestCase.installProduct('RevisionManager')
from OFS.PropertyManager import PropertyManager
from Products.RevisionManager.Properties import OFSPropertyAdapter
from Products.RevisionManager.interfaces.Properties import IPropertyAdapter

class MyPropContainer(PropertyManager):

    _properties = (
        {'id':'some',   'type': 'string', 'mode': 'w'},
        {'id':'any',    'type': 'string', 'mode': 'r'},
        {'id':'number', 'type': 'int',    'mode': 'wd'}
        )

    some = 'thing'
    any = 'body'

    def __init__(self):
        self.number = 234

class OFSPropertyAdapterTest(ZopeTestCase.ZopeTestCase):

    def afterSetUp(self):
        self.adapter = OFSPropertyAdapter()
        self.propcontainer = MyPropContainer()

    def afterClear(self):
        pass

    def test_verify_interface(self):
        """Verify interface implementation"""
        from Interface.Verify import verifyClass
        self.failUnless(verifyClass(IPropertyAdapter, OFSPropertyAdapter))

    def test_get_properties(self):
        """Test getProperties()"""
        adapter = self.adapter
        propcontainer = self.propcontainer
        expected = (
             {'id':'some',   'value':'thing', 'type': 'string'},
             {'id':'any',    'value':'body',  'type': 'string'},
             {'id':'number', 'value':234,     'type': 'int'}
            )
        result = adapter.getProperties(propcontainer)
        result = list(result)
        for prop in expected:
            self.failUnless(prop in result)
            result.remove(prop)
        self.failIf(result)

    def test_set_properties(self):
        """Test setProperties()"""
        adapter = self.adapter
        propcontainer = self.propcontainer
        expected = (
            {'id':'some',   'value':'time', 'type': 'string'},
            {'id':'any',    'value':789,    'type': 'int'   },
            {'id':'foo',    'value':'bar',  'type': 'string'},
            )
        adapter.setProperties(propcontainer, expected)
        result = adapter.getProperties(propcontainer)
        result = list(result)
        for prop in expected:
            self.failUnless(prop in result)
            result.remove(prop)
        self.failIf(result)

if __name__ == '__main__':
    framework(descriptions=1, verbosity=2)
else:
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(OFSPropertyAdapterTest))
        return suite

